package com.syjy.calculate.repository.repositoryImpl;

import com.syjy.calculate.config.StarterProperties;
import com.syjy.calculate.entity.tCalculationFormula;
import com.syjy.calculate.repository.CalculationFormulaRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.stereotype.Service;
import org.springframework.util.ClassUtils;

import javax.annotation.Resource;
import java.io.*;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class CalculationFormulaRepositoryImpl implements CalculationFormulaRepository {

    @Autowired
    StarterProperties properties;
    @Resource
    public JdbcTemplate jdbcTemplate;

    /**
     * 创建公式表
     */
    @Override
    public void createTable() {
        String config = properties.getDropTableFlag();
        // 如果不删除表，通过页面更改
        if ("false".equals(config)) {
            // 判断表是否存在
            String checkSql = "SHOW TABLES LIKE 't_calculation_formula'";
            Boolean created = jdbcTemplate.query(checkSql, null, null, new ResultSetExtractor<Boolean>() {
                @Override
                public Boolean extractData(ResultSet rs) throws SQLException, DataAccessException {
                    return rs.next();
                }
            });
            // 如果表存在，不对表做任何操作
            if (created) {
                return;
            }
        }
        try {
            // 获取初始化公式表sql
            String path = "sql/t_calculation_formula.sql";
            // 用ClassLoader获取jar包中的文件，获取文件流
            ClassLoader defaultClassLoader = ClassUtils.getDefaultClassLoader();
            assert defaultClassLoader != null;
            URL resource = defaultClassLoader.getResource(path);
            assert resource != null;
            InputStream inputStream = resource.openStream();
            // 循环将流放到StringBuilder中
            StringBuilder builder = new StringBuilder();
            int len;
            byte[] buf = new byte[1024];
            while ((len = inputStream.read(buf)) != -1) {
                builder.append(new String(buf, 0, len));
            }
            // 关闭流
            inputStream.close();
            // 执行sql文件
            jdbcTemplate.execute(builder.toString());

        } catch (DataAccessException e) {
            e.printStackTrace();
            log.info("删除/创建表错误：" + e.toString());
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 查询公式数据
     *
     * @param type         类型
     * @param provinceEnum
     * @return
     */
    @Override
    public List<tCalculationFormula> findByTypeAndProvince(String type, String provinceEnum) {
        // 模糊查询参数
        provinceEnum = "%" + provinceEnum + ";%";
        // 查询sql
        String sql = "SELECT * from t_calculation_formula where TYPE = ? and PROVINCE_ENUM like ?  ";
        // 根据类型和省调查询公式
        List<tCalculationFormula> calculationFormulaList = new ArrayList<>();
        try {
            calculationFormulaList = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(tCalculationFormula.class), type, provinceEnum);
        } catch (DataAccessException e) {
            e.printStackTrace();
        }
        return calculationFormulaList;
    }


}
